from typing import List, Optional, Union, Tuple

from arclet.alconna import CommandMeta
from nonebot import get_driver
from pydantic import BaseModel, Field, Extra, AnyHttpUrl

CMD_TYPE = Optional[Union[List[str], Tuple[bool, List[str]]]]


class Config(BaseModel, extra=Extra.ignore):
    nezha_api: AnyHttpUrl
    nezha_token: str

    nezha_use_start: bool = Field(default=True)
    # nezha_use_sep: bool = Field(default=False)
    # 是否使用紧凑模式(命令后是否可以紧跟参数)
    nezha_use_compact: bool = Field(default=True)

    nezha_at: bool = Field(default=False)
    nezha_replay: bool = Field(default=False)
    nezha_must_super_user: bool = Field(default=True)

    # nezha_prefix: List[str] = Field(default=["哪吒"])
    """
    None:不启用该插件
    List[str]:命令名
    Tuple[bool,List[str]]:是否只能管理使用,命令名
    """
    nezha_cmd_help: CMD_TYPE = Field(default=["哪吒帮助"])
    nezha_cmd_list: CMD_TYPE = Field(default=["vps列表"])
    nezha_cmd_details: CMD_TYPE = Field(default=["vps"])

    # 默认参数
    nezha_arg_default: Optional[Union[int, str]] = Field(default=None)

    nezha_template_server: str = Field(default=(
        "#$.name# #$.IfOnline#\n"
        "   ID: #$.id# TAG: #$.tag#\n"
        "   IPv4: #$.ipv4#\n"
        "   IPv6: #$.ipv6#"
    ))
    nezha_template_server_details: str = Field(default=(
        "#$.name# #$.IfOnline#\n"
        "   ID: #$.id# TAG: #$.tag#\n"
        "   负载: #$.status.Load1#,#$.status.Load5#,#$.status.Load15#\n"
        "   CPU: #$.CpuPercent#\n"
        "   内存: #$.MemoryPercent# 硬盘: #$.DiskPercent#\n"
        "   流量: #$.status.NetInTransfer#↓ ↑#$.status.NetOutTransfer#\n"
        "   IPv4: #$.ipv4#\n"
        "   IPv6: #$.ipv6#"
    ))
    # datetime.strftime(nezha_template_datetime)
    nezha_template_datetime: str = Field(default="%Y-%m-%d %H:%M:%S")
    nezha_template_datetime_placeholder: str = Field(default="无")
    nezha_template_online_offline: List[str] = Field(default=["在线", "离线"])

    # 浮点数小数位数、是否展示IP、是否省略IP中间部分、判断离线时间
    nezha_decimal_places: int = Field(default=1)
    nezha_show_ip: bool = Field(default=True)
    nezha_mask_ip: bool = Field(default=True)
    nezha_offline_time: float = Field(default=30)


config: Config = Config.parse_obj(get_driver().config)
_CM = CommandMeta(compact=config.nezha_use_compact, description="")

__all__ = ["Config", "config", "_CM", "CMD_TYPE"]



from json import loads
from pathlib import PurePosixPath
from typing import Literal, Any, Union, List, Optional, cast
from urllib.parse import urlparse, urlunparse

from nonebot import get_driver
from nonebot.internal.driver import Request, HTTPClientMixin
from pydantic import parse_obj_as

from .model import Server, ServerDetails

global_driver = get_driver()

assert isinstance(global_driver, HTTPClientMixin), "Driver has not HTTPClientMixin"

global_driver = cast(HTTPClientMixin, global_driver)


class API:

    def __init__(self, url: str, token: str):
        self.api = url
        self.token = token
        self.headers = {
            "Authorization": token
        }

    def get_api(self, path: str) -> str:
        u = urlparse(self.api)
        return urlunparse(u._replace(path=(PurePosixPath(u.path) / path).as_posix()))  # type:ignore

    async def call_api(
            self,
            path: str, method: Literal["GET", "PUT", "POST", "DELETE", "OPTIONS", "HEAD", "PATCH", "TRACE"], **data: Any
    ) -> Any:
        resp = await global_driver.request(Request(
            method, self.get_api(path),
            headers=self.headers,
            **({"params": data} if method == "GET" else {"json": data})
        ))

        res = loads(resp.content)

        assert res["code"] == 0, f"调用api失败<code:{res['code']}>: {res['message']}"

        return res.get("result")

    async def get_servers_by_tag(self, tag: Optional[str] = None) -> List[Server]:
        if tag is None:
            tag = ""
        return parse_obj_as(List[Server], await self.call_api("/api/v1/server/list", "GET", tag=tag))

    async def get_servers_details_by_tag(self, tag: Optional[str] = None) -> List[ServerDetails]:
        if tag is None:
            tag = ""
        return parse_obj_as(
            List[ServerDetails], await self.call_api("/api/v1/server/details", "GET", tag=tag)
        )

    async def get_servers_details_by_id(self, id_: Optional[Union[int, str]] = None) -> List[ServerDetails]:
        if id_ is None:
            id_ = ""
        return parse_obj_as(
            List[ServerDetails], await self.call_api("/api/v1/server/details", "GET", id=str(id_))
        )



from typing import Annotated, Union, Literal

from arclet.alconna import Args
from nonebot.plugin import PluginMetadata, inherit_supported_adapters, require

require("nonebot_plugin_alconna")

from nonebot_plugin_alconna import Match, AlconnaMatch

from .config import Config, config
from .tool import wrap_cmd, send, HELPS
from .render import render_list
from .api import API

api = API(config.nezha_api, config.nezha_token)


@wrap_cmd(config.nezha_cmd_help, Args["index?", int])
async def help_cmd_handle(index: Annotated[Match[int], AlconnaMatch("index")]):
    if index.available:
        await send(f"{index.result}.{HELPS[index.result]}")
    else:
        await send("\n".join(f"{i}.{h}" for i, h in enumerate(HELPS)))


@wrap_cmd(config.nezha_cmd_list, Args["tag?", Union[str, Literal["all"]], config.nezha_arg_default])
async def list_cmd_handle(tag: Annotated[Match[str], AlconnaMatch("tag")]):
    if tag.available:
        servers = await api.get_servers_by_tag("" if tag.result == "all" else tag.result)
    else:
        servers = await api.get_servers_by_tag()

    await send(render_list(config.nezha_template_server, servers) if len(servers) > 0 else "无服务器或未找到")


@wrap_cmd(config.nezha_cmd_details, Args["arg?", Union[int, str, Literal["all"]], config.nezha_arg_default])
async def details_cmd_handle(arg: Annotated[Match[Union[int, str]], AlconnaMatch("arg")]):
    if arg.available:
        if isinstance(arg.result, int) or "," in arg.result:
            servers = await api.get_servers_details_by_id(arg.result)
        else:
            servers = await api.get_servers_details_by_tag("" if arg.result == "all" else arg.result)
    else:
        servers = await api.get_servers_details_by_id()

    await send(render_list(config.nezha_template_server_details, servers) if len(servers) > 0 else "无服务器或未找到")


__plugin_meta__ = PluginMetadata(
    name="哪吒监控插件",
    description="哪吒监控插件，使用哪吒监控API。",
    usage="\n".join(f"{i}.{h}" for i, h in enumerate(HELPS)),
    type="application",
    homepage="https://github.com/eya46/nonebot_plugin_nezha",
    config=Config,
    supported_adapters=inherit_supported_adapters("nonebot_plugin_alconna"),
)
